using System;
using Syncfusion.Windows.Forms.Diagram;
using System.Runtime.Serialization;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;

namespace STN.PMA.UL
{
	/// <summary>
	/// Summary description for ConditionSysbol.
	/// </summary>
	/// 
	[Serializable()]
	public class clsConditionSymbol:clsSymbol
	{
		
		public RoundRect rect ;
		private Syncfusion.Windows.Forms.Diagram.Label lblDesc;
		private Syncfusion.Windows.Forms.Diagram.Label lblExpression;

		private const float DEFAULT_WIDTH=80;
		private const float DEFAULT_HEIGHT=20;

		public enum FieldsEnum {Description=1,Expression,None};
		public clsConditionSymbol()
		{
		
			 
			this.rect = new RoundRect(0, 0, DEFAULT_WIDTH, DEFAULT_HEIGHT);
			this.rect.Name="Rectangle";
			this.rect.FillStyle.Color = clsConst.TREE_CONDITION_COLOR;
			//rect.Rotate(45F);

			this.AppendChild(this.rect);	
			this.lblDesc = this.AddLabel("", BoxPosition.Center);
			this.lblDesc.Name="lblDesc";
			this.lblDesc.FontStyle.Style = System.Drawing.FontStyle.Regular; //Bold;
			this.lblDesc.FontStyle.Family="Tahoma";
			this.lblDesc.FontStyle.Size=6F;//10F;
			this.lblDesc.BackgroundStyle.Color=System.Drawing.Color.Transparent;
			this.lblDesc.ReadOnly=true;
			this.lblDesc.HorizontalAlignment=System.Drawing.StringAlignment.Center;
			this.lblDesc.VerticalAlignment=System.Drawing.StringAlignment.Center;
			this.lblDesc.Width=this.Width;
			this.lblDesc.Height=this.Height;
			this.lblDesc.Visible=true;

			this.lblExpression = this.AddLabel("", BoxPosition.Center);
			this.lblExpression.Name="lblExpression";
			this.lblExpression.FontStyle.Style = System.Drawing.FontStyle.Regular; //Bold;
			this.lblExpression.FontStyle.Family="Tahoma";
			this.lblExpression.FontStyle.Size=6F;//8.25F;

			this.lblExpression.BackgroundStyle.Color=System.Drawing.Color.Transparent;
			this.lblExpression.ReadOnly=true;
			this.lblExpression.HorizontalAlignment=System.Drawing.StringAlignment.Center;
			this.lblExpression.VerticalAlignment=System.Drawing.StringAlignment.Center;
			this.lblExpression.Width=this.Width;
			this.lblExpression.Height=this.Height;
			this.lblExpression.Visible=false;

		}
		// Copy Constructor
		public clsConditionSymbol(clsConditionSymbol src) : base(src)
		{
			this.rect = (RoundRect)this.Nodes.GetNodeByName("Rectangle");
			this.rect.FillStyle.Color = clsConst.TREE_CONDITION_COLOR;
			this.lblDesc  = (SymbolLabel)this.Nodes.GetNodeByName("lblDesc");
			this.lblExpression = (SymbolLabel)this.Nodes.GetNodeByName("lblExpression");
		
		}

		protected clsConditionSymbol(SerializationInfo info, StreamingContext context) : base(info, context)
		{

		}

		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
		}


		protected override void OnDeserialization(object sender)
		{
			base.OnDeserialization(sender);

			// Get references to the child objects
			this.rect = this.GetChildByName("Rectangle") as Syncfusion.Windows.Forms.Diagram.RoundRect;
			this.lblDesc = this.GetChildByName("lblDesc") as Syncfusion.Windows.Forms.Diagram.Label;
			this.lblExpression = this.GetChildByName("lblExpression") as Syncfusion.Windows.Forms.Diagram.Label;
		}


	
		public string Description
		{
			set 
			{
				this.lblDesc.Text=value;
			}
			get {return this.lblDesc.Text;}
		}

		public string Expression
		{
			set 
			{
				this.lblExpression.Text=value;
			}
			get {return this.lblExpression.Text;}
		}
		public FieldsEnum TextField
		{
			set 
			{
				switch (value)
				{
					case FieldsEnum.Description:
						this.lblDesc.Visible=true;
						this.lblExpression.Visible=false;
						break;
					case FieldsEnum.Expression:
						this.lblDesc.Visible=false;
						this.lblExpression.Visible=true;
						break;
					case FieldsEnum.None:
						this.lblDesc.Visible=false;
						this.lblExpression.Visible=false;
						break;
					default:
						break;
				}
			}
			get 
			{
				if (this.lblDesc.Visible) return FieldsEnum.Description;
				else if (this.lblExpression.Visible) return FieldsEnum.Expression;
				else return FieldsEnum.None;
			}

		}
		// Creates a new object that is a copy of the current instance.	
		// An override of the Clone method is required for clipboard copy/paste.
		public override object Clone()
		{
			return new clsConditionSymbol(this);
		}		

		private Color mOldColor=clsConst.TREE_NULL_COLOR;
		protected override void OnMouseEnter(NodeMouseEventArgs evtArgs)
		{
			//don't set color changes because this will caused Model.Modified=true
			// and also will add set property to redo list

//			// TODO:  Add ConditionSysbol.OnMouseEnter implementation
//			if (mOldColor==clsConst.TREE_NULL_COLOR) 
//			{
//				mOldColor=this.rect.FillStyle.Color;
//				this.rect.FillStyle.Color =clsConst.TREE_HOVER_COLOR;
//				Model md=this.Root as Model;
//				if (md!=null) md.UpdateViews();
//				
//			}
			
		}
	
		protected override void OnMouseLeave(NodeMouseEventArgs evtArgs)
		{
			//don't set color changes because this will caused Model.Modified=true
			// and also will add set property to redo list

//			// TODO:  Add ConditionSysbol.OnMouseLeave implementation
//			if (mOldColor!=clsConst.TREE_NULL_COLOR) 
//			{
//				this.rect.FillStyle.Color = mOldColor;
//				mOldColor=clsConst.TREE_NULL_COLOR;
//				Model md=this.Root as Model;
//				if (md!=null) md.UpdateViews();
//			}
		}
		public new void SetPathColor(Color PathColor)
		{
			mOldColor=PathColor;
			base.SetPathColor(PathColor);
		}


		protected override void OnBoundsChanged(BoundsEventArgs evtArgs)
		{
			this.lblDesc.Width=this.Width;
			this.lblDesc.Height=this.Height;
			this.lblExpression.Width=this.Width;
			this.lblExpression.Height=this.Height;

		}


		public override bool AcceptConnection(Port sourcePort, Port targetPort)
		{
			bool accept = false;

			// First, determine which port belongs to the foreign port container.

			Port foreignPort = null;

			if (this.Ports.Contains(sourcePort) || this.CenterPort == sourcePort)
			{
				foreignPort = targetPort;
			}
			else if (this.Ports.Contains(targetPort) || this.CenterPort == targetPort)
			{
				foreignPort = sourcePort;
			}

			if (foreignPort == null)
			{
				return false;  // This should never happen
			}

			
			// Now get the foreign port container and check to see if it is
			// a Link object. If so, then check to see if the foreign port
			// is the link's tail port or head port. If it's the tail port,
			// then always allow the connection, because a member symbol can
			// have any number of edges leaving it. If it's the head port,
			// then only allow the connection if the member symbol has no
			// other edges entering it yet.

			IPortContainer foreignContainer = foreignPort.Container;
			if (foreignContainer != null)
			{
				Link link = foreignContainer as Link;
				if (link != null)
				{
					if (foreignPort == link.TailPort)
					{
						//accept = true;

						IGraphNode curNode = this as IGraphNode;

						if (curNode != null)
						{
							ICollection edgesLeaving = curNode.EdgesLeaving;
							if (edgesLeaving.Count < 2)
							{
								accept = true;

								//check Yes/No
								if (edgesLeaving.Count==1)
								{
									Link linkAnother=null;
									//IEnumerator enumedgesEntering = curNode.EdgesEntering.GetEnumerator();
									IEnumerator enumedgesLeaving = this.EdgesLeaving.GetEnumerator();
									while (enumedgesLeaving.MoveNext())
									{
										Link linkTmp= enumedgesLeaving.Current as Link;
										if (linkTmp!=link) 
										{
											linkAnother=linkTmp;
											break; //only have one for our tree
										}
									}

									if (linkAnother!=null)
									{
										Syncfusion.Windows.Forms.Diagram.Label lblAnother=linkAnother.GetChildByName(clsConst.YES_NO_LABEL_NAME) as Syncfusion.Windows.Forms.Diagram.Label;
										
										Syncfusion.Windows.Forms.Diagram.Label lblYesNo=link.GetChildByName(clsConst.YES_NO_LABEL_NAME) as Syncfusion.Windows.Forms.Diagram.Label;
									
										if (lblAnother!=null)
										{
											if ( lblAnother.Text==clsConst.YES_VALUE)
											{
												lblYesNo.Text=clsConst.NO_VALUE;
												lblYesNo.FillStyle.Color=clsConst.TREE_NO_COLOR;
												link.LineStyle.LineColor=clsConst.TREE_NO_COLOR;
												link.FillStyle.Color=clsConst.TREE_NO_COLOR;
											}
											else
											{
												lblYesNo.Text=clsConst.YES_VALUE;  //default label
												lblYesNo.FillStyle.Color=clsConst.TREE_YES_COLOR;
												link.LineStyle.LineColor=clsConst.TREE_YES_COLOR;
												link.FillStyle.Color=clsConst.TREE_YES_COLOR;
											}
										}
									}
								}

							}
							else
							{
								//clsComm.MsgBox("A node can only have two children.");
							}
						}
					}
					else
					{
						IGraphNode curNode = this as IGraphNode;

						if (curNode != null)
						{
							ICollection edgesEntering = curNode.EdgesEntering;
							if (edgesEntering.Count == 0)
							{
								accept = true;
							}
							else
							{
								//clsComm.MsgBox("Node can only have one parent.");
							}
						}
					}
				}
			}

			return accept;
		}




	
	}
}
